package com.sq.customization.util;

public class ConditionChecker {

    public static void main(String[] args) {
        // 示例测试 1
        String condition1 = "%abc & ==5.0 & %xyz";
        String str1 = "This is abc xyz text";
        String range1 = "4.5-6.2";
        boolean result1 = checkCondition(condition1, str1, range1);
        System.out.println("条件 1: " + (result1 ? "符合" : "不符合"));

        // 示例测试 2
        String condition2 = "(%text & >5.5 & <=10.0) | %xyz";
        String str2 = "This is text";
        String range2 = "7-8"; // 空字符串
        boolean result2 = checkCondition(condition2, str2, range2);
        System.out.println("条件 2: " + (result2 ? "符合" : "不符合"));

        // 示例测试 3
        String condition3 = "<3.0 &==";
        String str3 = "";
        String range3 = "";
        boolean result3 = checkCondition(condition3, str3, range3);
        System.out.println("条件 3: " + (result3 ? "符合" : "不符合"));

        // 示例测试 4
        String condition4 = "%abcg|%t";
        String str4 = "abc text";
        String range4 = "";
        boolean result4 = checkCondition(condition4, str4, range4);
        System.out.println("条件 4: " + (result4 ? "符合" : "不符合"));
    }

    /**
     * 检查条件是否符合
     */
    public static boolean checkCondition(String condition, String str, String range) {
        condition = condition.trim();

        // 解析范围
        double[] rangeBounds = parseRange(range);
        double min = rangeBounds != null ? rangeBounds[0] : Double.NaN; // 用 NaN 表示无效范围
        double max = rangeBounds != null ? rangeBounds[1] : Double.NaN;

        // 处理括号
        if (condition.startsWith("(")) {
            int parenthesesLevel = 1;
            int endIndex = -1;
            for (int i = 1; i < condition.length(); i++) {
                if (condition.charAt(i) == '(') parenthesesLevel++;
                else if (condition.charAt(i) == ')') parenthesesLevel--;
                if (parenthesesLevel == 0) {
                    endIndex = i;
                    break;
                }
            }
            if (endIndex != -1 && endIndex == condition.length() - 1) {
                return checkCondition(condition.substring(1, endIndex), str, range);
            }
        }

        // 处理多个 & 和 | 运算符
        int[] operatorIndices = findAllOuterOperators(condition);
        if (operatorIndices.length > 0) {
            String[] parts = splitCondition(condition, operatorIndices);
            char[] operators = getOperators(condition, operatorIndices);

            boolean result = checkCondition(parts[0], str, range);
            int partIndex = 1;
            for (int i = 0; i < operators.length; i++) {
                boolean nextResult = checkCondition(parts[partIndex++], str, range);
                if (operators[i] == '&') {
                    result = result && nextResult;
                } else if (operators[i] == '|') {
                    result = result || nextResult;
                }
            }
            return result;
        }

        // 处理单个条件
        return checkSingleCondition(condition, str, min, max);
    }

    /**
     * 解析范围字符串，支持小数
     */
    private static double[] parseRange(String range) {
        if (range == null || range.isEmpty()) {
            return null; // 空字符串返回 null，表示无有效范围
        }
        try {
            if (range.contains("-")) {
                String[] parts = range.split("-");
                double start = Double.parseDouble(parts[0].trim());
                double end = Double.parseDouble(parts[1].trim());
                return new double[]{start, end};
            } else if (range.contains("±")) {
                String[] parts = range.split("±");
                double center = Double.parseDouble(parts[0].trim());
                double delta = Double.parseDouble(parts[1].trim());
                return new double[]{center - delta, center + delta};
            }
        } catch (NumberFormatException | ArrayIndexOutOfBoundsException e) {
            throw new IllegalArgumentException("无效的范围格式: " + range + "，应为 'x-y' 或 'x±y'");
        }
        return null;
    }

    /**
     * 查找所有最外层的 & 和 | 运算符
     */
    private static int[] findAllOuterOperators(String condition) {
        int parenthesesLevel = 0;
        java.util.List<Integer> indices = new java.util.ArrayList<>();
        for (int i = 0; i < condition.length(); i++) {
            char c = condition.charAt(i);
            if (c == '(') parenthesesLevel++;
            else if (c == ')') parenthesesLevel--;
            else if (parenthesesLevel == 0 && (c == '&' || c == '|')) indices.add(i);
        }
        return indices.stream().mapToInt(Integer::intValue).toArray();
    }

    /**
     * 根据运算符位置分割条件
     */
    private static String[] splitCondition(String condition, int[] operatorIndices) {
        String[] parts = new String[operatorIndices.length + 1];
        int start = 0;
        for (int i = 0; i < operatorIndices.length; i++) {
            parts[i] = condition.substring(start, operatorIndices[i]).trim();
            start = operatorIndices[i] + 1;
        }
        parts[operatorIndices.length] = condition.substring(start).trim();
        return parts;
    }

    /**
     * 获取所有运算符
     */
    private static char[] getOperators(String condition, int[] operatorIndices) {
        char[] operators = new char[operatorIndices.length];
        for (int i = 0; i < operatorIndices.length; i++) {
            operators[i] = condition.charAt(operatorIndices[i]);
        }
        return operators;
    }

    /**
     * 检查单个条件，使用范围的最小值和最大值，支持小数
     */
    private static boolean checkSingleCondition(String condition, String str, double min, double max) {
        condition = condition.trim();

        if (condition.startsWith("!%")) {
            String substring = condition.substring(2);
            return !str.contains(substring);
        } else if (condition.startsWith("%")) {
            String substring = condition.substring(1);
            return str.contains(substring);
        } else if (condition.startsWith(">=")) {
            if (Double.isNaN(min)) return false; // 空范围，不符合数字比较
            double value = Double.parseDouble(condition.substring(2));
            return min >= value;
        } else if (condition.startsWith("<=")) {
            if (Double.isNaN(max)) return false; // 空范围，不符合数字比较
            double value = Double.parseDouble(condition.substring(2));
            return max <= value;
        } else if (condition.startsWith(">")) {
            if (Double.isNaN(min)) return false; // 空范围，不符合数字比较
            double value = Double.parseDouble(condition.substring(1));
            return min > value;
        } else if (condition.startsWith("<")) {
            if (Double.isNaN(max)) return false; // 空范围，不符合数字比较
            double value = Double.parseDouble(condition.substring(1));
            return max < value;
        } else if (condition.startsWith("==")) {
            String operand = condition.substring(2);
            try {
                if (Double.isNaN(min) || Double.isNaN(max)) return false; // 空范围，不符合数字比较
                double value = Double.parseDouble(operand);
                return min == value && max == value;
            } catch (NumberFormatException e) {
                return str.equals(operand);
            }
        } else if (condition.startsWith("!=")) {
            String operand = condition.substring(2);
            try {
                if (Double.isNaN(min) || Double.isNaN(max)) return false; // 空范围，不符合数字比较
                double value = Double.parseDouble(operand);
                return min != value || max != value;
            } catch (NumberFormatException e) {
                return !str.equals(operand);
            }
        }

        return false;
    }
}
